@@ -33,7 +33,7 @@ $c->pg_connect[] = "dbname=davical user=davical_app";
 
 /**
 * default: true
-* If true, then TODO requested  from someone other than the admmin or owner
+* If true, then VTODO requested  from someone other than the admmin or owner
 * of a calendar will not get any answer.  Often these todo are only relevant
 * to the owner, but in some shared calendar situations they might not be in
 * which case you should set this to false.
@@ -188,6 +188,16 @@ $c->collections_always_exist = false;
 $c->schedule_private_key = 'PRIVATE-KEY-BASE-64-DATA';
 */
 
+/*
+* External subscription (BIND) minimum refresh interval
+* Required if you want to enable remote binding ( webcal subscriptions )
+* Default: none
+*/  
+/*
+$c->external_refresh = 60;
+*/
+
+
 
 
 /***************************************************************************
@@ -204,7 +214,7 @@ $c->schedule_private_key = 'PRIVATE-KEY-BASE-64-DATA';
 *   call_user_func( $c->authenticate_hook['call'], $username, $password )
 *
 * The login mecanism is made in 2 places:
-*  - for the web interface in: index.php that calls RSCDSSession.php that extends
+*  - for the web interface in: index.php that calls DAViCalSession.php that extends
 *    Session.php (from AWL libraries)
 *  - for the caldav client in: caldav.php that calls BasicAuthSession.php
 * Both Session.php and BasicAuthSession.php check against the
@@ -354,7 +364,7 @@ $c->schedule_private_key = 'PRIVATE-KEY-BASE-64-DATA';
 * are supported on your system.
 *
 * People interested in providing new translations are directed to the Wiki:
-*   http://rscds.sourceforge.net/moin/Translating_DAViCal
+*   http://wiki.davical.org/w/Translating_DAViCal
 **/
 // $c->default_locale = "en_NZ";
 
