@@ -26,6 +26,7 @@ function early_exception_handler($e) {
   foreach( $trace AS $k => $v ) {
     printf( "%s[%d] %s%s%s()\n", $v['file'], $v['line'], (isset($v['class'])?$v['class']:''), (isset($v['type'])?$v['type']:''), (isset($v['function'])?$v['function']:'') );
   }
+  @ob_flush();
 }
 set_exception_handler('early_exception_handler');
 
@@ -89,7 +90,7 @@ if ( ! @include_once('AWLUtilities.php') ) {
   }
   if ( ! @include_once('AWLUtilities.php') ) {
     echo "Could not find the AWL libraries. Are they installed? Check your include_path in php.ini!\n";
-    exit;
+    @ob_flush(); exit(0);
   }
 }
 
@@ -145,7 +146,7 @@ else if ( @file_exists('config/config.php') ) {
 }
 else {
   include('davical_configuration_missing.php');
-  exit;
+  @ob_flush();  exit(0);
 }
 $config_warnings = trim(ob_get_contents());
 ob_end_clean();
@@ -174,8 +175,8 @@ init_gettext( 'davical', $c->locale_path );
 *
 */
 $c->code_version = 0;
-$c->want_awl_version = '0.49';
-$c->version_string = '0.9.9.7'; // The actual version # is replaced into that during the build /release process
+$c->want_awl_version = '0.51';
+$c->version_string = '1.0.2'; // The actual version # is replaced into that during the build /release process
 if ( isset($c->version_string) && preg_match( '/(\d+)\.(\d+)\.(\d+)(.*)/', $c->version_string, $matches) ) {
   $c->code_major = $matches[1];
   $c->code_minor = $matches[2];
