@@ -11,6 +11,7 @@
 
 require_once('AwlCache.php');
 require_once('AwlQuery.php');
+require_once('DAVPrincipal.php');
 require_once('DAVTicket.php');
 require_once('iCalendar.php');
 
@@ -282,6 +283,17 @@ class DAVResource
             $this->resource->location = null;
             $this->resource->url = null;
           }
+          else if ( isset($c->hide_alarm) && $c->hide_alarm && !$this->HavePrivilegeTo('write') ) {
+            $vcal1 = new iCalComponent($this->resource->caldav_data);
+            $comps = $vcal1->GetComponents();
+            $vcal2 = new iCalComponent();
+            $vcal2->VCalendar();
+            foreach( $comps AS $comp ) {
+              $comp->ClearComponents('VALARM');
+              $vcal2->AddComponent($comp);
+            }
+            $this->resource->caldav_data = $vcal2->Render();
+          }
         }
         else if ( strtoupper(substr($this->resource->caldav_data,0,11)) == 'BEGIN:VCARD' ) {
           $this->contenttype = 'text/vcard';
@@ -439,7 +451,6 @@ EOSQL;
         else
           $this->collection->type = 'collection';
         if ( strlen($row->external_url) > 8 ) {
-          $this->collection->bound_from = $row->external_url;
           $this->_is_external = true;
           if ( $row->external_type == 'calendar' )
             $this->collection->type = 'calendar';
@@ -627,7 +638,7 @@ EOQRY;
   protected function FetchPrivileges() {
     global $session, $request;
 
-    if ( $this->dav_name == '/' || $this->dav_name == '' ) {
+    if ( $this->dav_name == '/' || $this->dav_name == '' || $this->_is_external ) {
       $this->privileges = (1 | 16 | 32); // read + read-acl + read-current-user-privilege-set
       dbg_error_log( 'DAVResource', ':FetchPrivileges: Read permissions for user accessing /' );
       return;
@@ -699,7 +710,7 @@ EOQRY;
 
   
   /**
-  * Fetch the parent to this resource.
+  * Fetch the parent to this resource. This is deprecated - use GetParentContainer() instead.
   */
   function FetchParentContainer() {
     deprecated('DAVResource::FetchParentContainer');
@@ -1047,8 +1058,8 @@ EOQRY;
 
 
   /**
-  * Checks whether this resource is a calendar
-  * @param string $type The type of scheduling collection, 'read', 'write' or 'any'
+  * Checks whether this resource is a scheduling inbox/outbox collection
+  * @param string $type The type of scheduling collection, 'inbox', 'outbox' or 'any'
   */
   function IsSchedulingCollection( $type = 'any' ) {
     if ( $this->_is_collection && preg_match( '{schedule-(inbox|outbox)}', $this->collection->type, $matches ) ) {
@@ -1058,6 +1069,18 @@ EOQRY;
   }
 
 
+  /**
+  * Checks whether this resource is IN a scheduling inbox/outbox collection
+  * @param string $type The type of scheduling collection, 'inbox', 'outbox' or 'any'
+  */
+  function IsInSchedulingCollection( $type = 'any' ) {
+    if ( !$this->_is_collection && preg_match( '{schedule-(inbox|outbox)}', $this->collection->type, $matches ) ) {
+      return ($type == 'any' || $type == $matches[1]);
+    }
+    return false;
+  }
+
+
   /**
   * Checks whether this resource is an addressbook
   */
@@ -1244,6 +1267,14 @@ EOQRY;
   }
 
 
+  /**
+  * Checks whether the target collection is for public events only
+  */
+  function IsPublicOnly() {
+    return ( isset($this->collection->publicly_events_only) && $this->collection->publicly_events_only == 't' );
+  }
+
+
   /**
   * Return the type of whatever contains this resource, or would if it existed.
   */
@@ -1370,6 +1401,11 @@ EOQRY;
         return clone($this->resource);
         break;
 
+      case 'dav-data':
+        if ( !isset($this->resource) ) $this->FetchResource();
+        return $this->resource->caldav_data;
+        break;
+        
       case 'principal':
         if ( !isset($this->principal) ) $this->FetchPrincipal();
         return clone($this->principal);
@@ -1547,8 +1583,13 @@ EOQRY;
 
       case 'DAV::owner':
         // The principal-URL of the owner
-        if ( !isset($this->principal) ) $this->FetchPrincipal();
-        $reply->DAVElement( $prop, 'owner', $reply->href( $this->principal->url() ) );
+        if ( $this->IsExternal() ){
+          $reply->DAVElement( $prop, 'owner', $reply->href( $this->collection->bound_from ) );
+        }
+        else {
+          if ( !isset($this->principal) ) $this->FetchPrincipal();
+          $reply->DAVElement( $prop, 'owner', $reply->href( $this->principal->url() ) );
+        }
         break;
 
       // Empty tag responses.
