@@ -137,19 +137,29 @@ class Principal {
         break;
     }
 
-    
+
+    /**
+     * There are some values we can construct on the basis of the constructor value.
+     */
+    switch ( $type ) {
+      case 'user_no':        $this->user_no = $value;          break;
+      case 'principal_id':   $this->principal_id = $value;     break;
+      case 'email':          $this->email = $value;            break;
+      case 'username':       $this->username = $value;         break;
+      default:
+        throw new Exception('Can only retrieve a Principal by user_no,principal_id,username or email address');
+    }
+
     $cache = new AwlCache();
     if ( $use_cache && isset($session->principal_id) ) {
       switch ( $type ) {
         case 'user_no':
-          $this->user_no = $value;
           if ( isset(self::$byUserno[$value]) ) {
             $type = 'username';
             $value = self::$byUserno[$value];
           }
           break;
         case 'principal_id':
-          $this->principal_id = $value;
           if ( isset(self::$byId[$value]) ) {
             $type = 'username';
             $value = self::$byId[$value];
@@ -157,16 +167,11 @@ class Principal {
           break;
         case 'email':
           $this->by_email = true;
-          $this->email = $value;
           if ( isset(self::$byEmail[$value]) ) {
             $type = 'username';
             $value = self::$byEmail[$value];
           }
           break;
-        case 'username':
-          break;
-        default:
-          throw new Exception('Can only retrieve a Principal by user_no,principal_id,username or email address');
       }
 
       if ( $type == 'username' ) {
@@ -200,7 +205,7 @@ class Principal {
     $sql .= 'FROM dav_principal WHERE ';
     switch ( $type ) {
       case 'username':
-        $sql .= 'lower(username)=lower(:param)';
+        $sql .= 'lower(username)=lower(text(:param))';
         break;
       case 'user_no':
         $sql .= 'user_no=:param';
@@ -260,13 +265,12 @@ class Principal {
   private function assignGuestValues() {
     $this->user_no = -1;
     $this->exists = false;
-    $this->username = translate('unauthenticated');
+    if ( empty($this->username) ) $this->username = translate('unauthenticated');
     $this->fullname = $this->displayname = translate('Unauthenticated User');
     $this->email = false;
     $this->is_principal = true;
     $this->is_calendar = false;
     $this->principal_id = -1;
-    $this->dav_name = '/';
     $this->privileges = $this->default_privileges = 0;    
   }
 
@@ -528,14 +532,19 @@ class Principal {
     $sql_params = array();
     foreach( self::updateableFields() AS $k ) {
       if ( !isset($field_values->{$k}) && !isset($this->{$k}) ) continue;
+      $param_name = ':'.$k;
+      $sql_params[$param_name] = (isset($field_values->{$k}) ? $field_values->{$k} : $this->{$k});  
+      if ( $k == 'default_privileges' ) {
+        $sql_params[$param_name] = sprintf('%024s',$sql_params[$param_name]);
+        $param_name = 'cast('.$param_name.' as text)::BIT(24)';
+      }
       if ( $inserting ) {
+        $param_names[] = $param_name;
         $insert_fields[] = $k;
-        $param_names[] = ':'.$k . ($k == 'default_privileges' ? '::BIT(24)' : '');
       }
       else {
-        $update_list[] = $k.'=:'.($k == 'default_privileges' ? '::BIT(24)' : '');
+        $update_list[] = $k.'='.$param_name;
       }
-      $sql_params[':'.$k] = (isset($field_values->{$k}) ? $field_values->{$k} : $this->{$k});  
     }
 
     if ( $inserting && isset(self::$db_mandatory_fields) ) {
