@@ -343,6 +343,9 @@ class RepeatRuleDateTime extends DateTime {
 //    printf( "Modify '%s' by: >>%s<<\n", $this->__toString(), $interval );
 //    print_r($this);
     if ( !isset($interval) || $interval == '' ) $interval = '1 day';
+    if ( parent::format('d') > 28 && strstr($interval,'month') !== false ) {
+      $this->setDate(null,null,28);
+    }
     parent::modify($interval);
     return $this->__toString();
   }
@@ -431,10 +434,36 @@ class RepeatRuleDateTime extends DateTime {
   }
 
 
+  /**
+   * Returns a 1 if this year is a leap year, otherwise a 0
+   * @param int $year The year we are quizzical about.
+   * @return 1 if this is a leap year, 0 otherwise
+   */
+  public static function hasLeapDay($year) {
+    if ( ($year % 4) == 0 && (($year % 100) != 0 || ($year % 400) == 0) ) return 1;
+    return 0;
+  }
+
+  /**
+   * Returns the number of days in a year/month pair
+   * @param int $year
+   * @param int $month
+   * @return int the number of days in the month
+   */
+  public static function daysInMonth( $year, $month ) {
+    if ($month == 4 || $month == 6 || $month == 9 || $month == 11) return 30;
+    else if ($month != 2) return 31;
+    return 28 + RepeatRuleDateTime::hasLeapDay($year);
+  }
+
+  
   function setDate( $year=null, $month=null, $day=null ) {
     if ( !isset($year) )  $year  = parent::format('Y');
     if ( !isset($month) ) $month = parent::format('m');
     if ( !isset($day) )   $day   = parent::format('d');
+    if ( $day < 0 ) {
+      $day += RepeatRuleDateTime::daysInMonth($year, $month) + 1;
+    }
     parent::setDate( $year , $month , $day );
     return $this;
   }
@@ -484,6 +513,73 @@ class RepeatRuleDateTime extends DateTime {
 }
 
 
+/**
+ * This class is used to hold a pair of dates defining a range.  The range may be open-ended by including
+ * a null for one end or the other, or both.
+ *  
+ * @author Andrew McMillan <andrew@mcmillan.net.nz>
+ */
+class RepeatRuleDateRange {
+  public $from;
+  public $until;
+
+  /**
+   * Construct a new RepeatRuleDateRange which will be the range between $date1 and $date2. The earliest of the two
+   * dates will be used as the start of the period, the latest as the end.  If one of the dates is null then the order
+   * of the parameters is significant, with the null treated as -infinity if it is first, or +infinity if it is second.
+   * If both parameters are null then the range is from -infinity to +infinity.
+   *  
+   * @param RepeatRuleDateTime $date1
+   * @param RepeatRuleDateTime $date2
+   */
+  function __construct( $date1, $date2 ) {
+    if ( $date1 != null && $date2 != null && $date1 > $date2 )  {
+      $this->from = $date2;
+      $this->until = $date1;
+    }
+    else {
+      $this->from = $date1;
+      $this->until = $date2;
+    }
+  }
+
+  /**
+   * Assess whether this range overlaps the supplied range.  null values are treated as infinity.
+   * @param RepeatRuleDateRange $other
+   * @return boolean
+   */
+  function overlaps( RepeatRuleDateRange $other ) {
+    if ( ($this->until == null && $this->from == null) || ($other->until == null && $other->from == null ) ) return true;
+    if ( $this->until == null && $other->until == null ) return true;
+    if ( $this->from == null && $other->from == null ) return true;
+
+    if ( $this->until == null ) return ($other->until > $this->from);
+    if ( $this->from == null ) return ($other->from < $this->until);
+    if ( $other->until == null ) return ($this->until > $other->from);
+    if ( $other->from == null ) return ($thi->from < $other->until);
+
+    return !( $this->until < $other->from || $this->from > $other->until );
+  }
+
+  /**
+   * Get an Rfc5545Duration from this date range.  If the from date is null it will be null.
+   * If the until date is null the duration will either be 1 day (if the from is a date) or 0 otherwise.
+   *   
+   * @return NULL|Rfc5545Duration
+   */
+  function getDuration() {
+    if ( !isset($this->from) ) return null;
+    if ( $this->from->isDate() && !isset($this->until) )
+      $duration = 'P1D';
+    else if ( !isset($this->until) )
+      $duration = 'P0D';
+    else
+      $duration = ( $this->until->epoch() - $this->from->epoch() );
+    return new Rfc5545Duration( $duration );
+  }
+}
+
+
 /**
  * This class is an implementation of RRULE parsing and expansion, as per RFC5545.  It should be reasonably
  * complete, except that it does not handle changing the WKST - there may be a few errors in unusual rules
@@ -565,6 +661,15 @@ class RepeatRule {
   }
 
 
+  /**
+   * If this repeat rule has an UNTIL= or COUNT= then we can know it will end. Eventually.
+   * @return boolean Whether or not one of these properties is present.
+   */
+  public function hasLimitedOccurrences() {
+    return ( isset($this->count) || isset($this->until) );    
+  }
+
+
   public function set_timezone( $tzstring ) {
     $this->base->setTimezone(new DateTimeZone($tzstring));
   }
@@ -577,7 +682,7 @@ class RepeatRule {
     $this->finished = false;
   }
 
-
+  
   public function rewind() {
     $this->position = -1;
   }
@@ -692,7 +797,10 @@ class RepeatRule {
           $position++;
           $this->instances[$position] = $instance;
           if ( DEBUG_RRULE ) printf( "Added date %s into position %d in current set\n", $instance->format('c'), $position );
-          if ( isset($this->count) && ($position + 1) >= $this->count ) $this->finished = true;
+          if ( isset($this->count) && ($position + 1) >= $this->count ) {
+            $this->finished = true;
+            return;
+          }
         }
       }
     }
@@ -749,7 +857,9 @@ class RepeatRule {
     $this->current_set = array();
     foreach( $instances AS $k => $instance ) {
       foreach( $this->bymonthday AS $k => $monthday ) {
-        $this->current_set[] = $this->date_mask( clone($instance), null, null, $monthday, null, null, null);
+        $expanded = $this->date_mask( clone($instance), null, null, $monthday, null, null, null);
+        if ( DEBUG_RRULE ) printf( "Expanded BYMONTHDAY $monthday into date %s from %s\n", $expanded->format('c'), $instance->format('c') );
+        $this->current_set[] = $expanded;
       }
     }
   }
@@ -985,6 +1095,7 @@ class RepeatRule {
 }
 
 
+
 require_once("vComponent.php");
 
 /**
@@ -1070,11 +1181,13 @@ function expand_event_instances( $vResource, $range_start = null, $range_end = n
   global $c;
   $components = $vResource->GetComponents();
 
-  if ( !isset($range_start) ) { $range_start = new RepeatRuleDateTime(); $range_start->modify('-6 weeks'); }
-  if ( !isset($range_end) )   { $range_end   = clone($range_start);      $range_end->modify('+6 months');  }
+  if ( empty($range_start) ) { $range_start = new RepeatRuleDateTime(); $range_start->modify('-6 weeks'); }
+  if ( empty($range_end) )   {
+      $range_end   = clone($range_start);
+      $range_end->modify('+6 months');  
+  }
 
   $new_components = array();
-  $result_limit = 1000;
   $instances = array();
   $expand = false;
   $dtstart = null;
@@ -1270,3 +1383,118 @@ function expand_event_instances( $vResource, $range_start = null, $range_end = n
 
   return $vResource;
 }
+
+
+/**
+ * Return a date range for this component.
+ * @param vComponent $comp
+ * @throws Exception (1) When DTSTART is not present but the RFC says MUST and (2) when we get an unsupported component
+ * @return RepeatRuleDateRange
+ */
+function getComponentRange(vComponent $comp) {
+  $dtstart_prop = $comp->GetProperty('DTSTART');
+  $duration_prop = $comp->GetProperty('DURATION');
+  if ( isset($duration_prop) ) {
+    if ( !isset($dtstart_prop) ) throw new Exception('Invalid '.$comp->GetType().' containing DURATION without DTSTART', 0);
+    $dtstart = new RepeatRuleDateTime($dtstart_prop);
+    $dtend = clone($dtstart);
+    $dtend->modify(new Rfc5545Duration($duration_prop->Value()));
+  }
+  else {
+    $completed_prop = null;
+    switch ( $comp->GetType() ) {
+      case 'VEVENT':
+        if ( !isset($dtstart_prop) ) throw new Exception('Invalid VEVENT without DTSTART', 0);
+        $dtend_prop = $comp->GetProperty('DTEND');
+        break;
+      case 'VTODO':
+        $completed_prop = $comp->GetProperty('COMPLETED');
+        $dtend_prop = $comp->GetProperty('DUE');
+        break;
+      case 'VJOURNAL':
+        if ( !isset($dtstart_prop) )
+          $dtstart_prop = $comp->GetProperty('DTSTAMP');
+        $dtend_prop = $dtstart_prop;
+      default:
+        throw new Exception('getComponentRange cannot handle "'.$comp->GetType().'" components', 0);
+    }
+  
+    if ( isset($dtstart_prop) )
+      $dtstart = new RepeatRuleDateTime($dtstart_prop);
+    else 
+      $dtstart = null;
+  
+    if ( isset($dtend_prop) )
+      $dtend = new RepeatRuleDateTime($dtend_prop);
+    else
+      $dtend = null;
+
+    if ( isset($completed_prop) ) {
+      $completed = new RepeatRuleDateTime($completed_prop);
+      if ( !isset($dtstart) || (isset($dtstart) && $completed < $dtstart) ) $dtstart = $completed;
+      if ( !isset($dtend) || (isset($dtend) && $completed > $dtend) ) $dtend = $completed;
+    }
+  }
+  return new RepeatRuleDateRange($dtstart, $dtend);
+}
+
+/**
+* Return a RepeatRuleDateRange from the earliest start to the latest end of the event.
+* 
+* @todo: This should probably be made part of the VCalendar object when we move the RRule.php into AWL.
+*
+* @param object $vResource A vComponent which is a VCALENDAR containing components needing expansion
+* @return RepeatRuleDateRange Representing the range of time covered by the event. 
+*/
+function getVCalendarRange( $vResource ) {
+  global $c;
+  $components = $vResource->GetComponents();
+
+  $dtstart = null;
+  $duration = null;
+  $earliest_start = null;
+  $latest_end = null;
+  $has_repeats = false;
+  foreach( $components AS $k => $comp ) {
+    if ( $comp->GetType() == 'VTIMEZONE' ) continue;
+    $range = getComponentRange($comp);
+    $dtstart = $range->from;
+    if ( !isset($dtstart) ) continue;
+    $duration = $range->getDuration();
+
+    $rrule = $comp->GetProperty('RRULE');
+    $limited_occurrences = true;
+    if ( isset($rrule) ) {
+      $rule = new RepeatRule($dtstart, $rrule);
+      $limited_occurrences  = $rule->hasLimitedOccurrences();
+    }
+    
+    if ( $limited_occurrences ) {
+      $instances = array();
+      $instances[$dtstart->FloatOrUTC()] = $dtstart;
+      if ( !isset($range_end) ) {
+        $range_end   = new RepeatRuleDateTime();
+        $range_end->modify('+150 years');
+      }  
+      $instances += rrule_expand($dtstart, 'RRULE', $comp, $range_end);
+      $instances += rdate_expand($dtstart, 'RDATE', $comp, $range_end);
+      foreach ( rdate_expand($dtstart, 'EXDATE', $comp, $range_end) AS $k => $v ) {
+        unset($instances[$k]);
+      }
+      $instances = array_keys($instances);
+      asort($instances);
+      $first = new RepeatRuleDateTime($instances[0]);
+      $last = new RepeatRuleDateTime($instances[count($instances)-1]);
+      $last->modify($duration);
+      if ( empty($earliest_start) || $first < $earliest_start )  $earliest_start = $first;
+      if ( empty($latest_end) || $last > $latest_end )           $latest_end = $last;
+    }
+    else {
+      if ( empty($earliest_start) || $dtstart < $earliest_start ) $earliest_start = $dtstart;
+      $latest_end = null;
+      break;
+    }
+  }
+
+  return new RepeatRuleDateRange($earliest_start, $latest_end );
+}
