@@ -376,8 +376,8 @@ function build_privileges_html( $ed, $fname ) {
   $btn_rw  = htmlspecialchars(translate('Read/Write'));      $btn_rw_title = htmlspecialchars(translate('Set read+write privileges'));
   $btn_read = htmlspecialchars(translate('Read'));           $btn_read_title = htmlspecialchars(translate('Set read privileges'));
   $btn_fb = htmlspecialchars(translate('Free/Busy'));        $btn_fb_title = htmlspecialchars(translate('Set free/busy privileges'));
-  $btn_sd = htmlspecialchars(translate('Schedule Deliver')); $btn_sd_title = htmlspecialchars(translate('Set schedule-deliver privileges'));
-  $btn_ss = htmlspecialchars(translate('Schedule Send'));    $btn_ss_title = htmlspecialchars(translate('Set schedule-deliver privileges'));
+  $btn_sd = htmlspecialchars(translate('Schedule Deliver')); $btn_sd_title = htmlspecialchars(translate('Privileges to allow delivery of scheduling messages'));
+  $btn_ss = htmlspecialchars(translate('Schedule Send'));    $btn_ss_title = htmlspecialchars(translate('Privileges to delegate scheduling decisions'));
   
   $privs_dec = bindec($ed->Value($fname));
   $privileges_set = sprintf('<div id="privileges"><input type="hidden" name="%s[fake_privilege_for_input]" value="0">%s', $fname, "\n");
@@ -794,7 +794,7 @@ function access_ticket_browser() {
     $ticket_id = substr( str_replace('/', '', str_replace('+', '',base64_encode(sha1(date('r') .rand(0,2100000000) . microtime(true),true)))), 7, 8);
     $extra_row = array( 'ticket_id' => $ticket_id,
                         'expires' => date( 'Y-m-d', time() + (86400 * 31) ),
-                        'target' => '/'.$editor->Value('username').'/home/'
+                        'target' => '/'.$editor->Value('username').'/'.$c->home_calendar_name.'/'
                       );
     $browser->MatchedRow('ticket_id', $ticket_id, 'edit_ticket_row');
     $browser->AddRow($extra_row);
